@@ -1,6 +1,5 @@
 import contextlib
 import json
-import keyring
 import os
 import os.path as op
 import random
@@ -10,10 +9,17 @@ import time
 from functools import lru_cache
 from pathlib import Path
 
+from keyring.core import get_keyring, load_config, load_env
+from keyring.backend import get_all_keyring
+from keyring.errors import KeyringError
+from keyring.util.platform_ import config_root
+from keyrings.alt.file import EncryptedKeyring
+
 import girder_client as gcl
 
 from . import get_logger
 from .exceptions import LockingError
+from .support.ui import askyesno
 from .utils import ensure_datetime, flattened, flatten, remap_dict
 from .consts import known_instances_rev, MAX_CHUNK_SIZE
 
@@ -148,7 +154,7 @@ class GirderCli(gcl.GirderClient):
             raise NotImplementedError("TODO client name derivation for keyring")
 
         app_id = "dandi-girder-{}".format(client_name)
-        api_key = keyring.get_password(app_id, "key")
+        keyring_backend, api_key = keyring_lookup(app_id, "key")
         # the dance about rejected authentication etc
         while True:
             if not api_key:
@@ -157,7 +163,7 @@ class GirderCli(gcl.GirderClient):
                     "Account/API keys "
                     "in Girder) for {}: ".format(client_name)
                 )
-                keyring.set_password(app_id, "key", api_key)
+                keyring_backend.set_password(app_id, "key", api_key)
                 lgr.debug("Stored key in keyring")
 
             try:
@@ -286,7 +292,7 @@ class GirderCli(gcl.GirderClient):
         elif a["type"] == "item":
             file_recs = list(self.listFile(g["_id"]))
             if len(file_recs) > 1:
-                lgr.warning("Multiple files for one item found; using oldest one")
+                lgr.warning("Multiple files found for %s; using oldest one", a["path"])
                 file_recs = [
                     min(file_recs, key=lambda fr: ensure_datetime(fr["created"]))
                 ]
@@ -845,3 +851,86 @@ class TQDMProgressReporter(object):
             self._pbar.clear()  # remove from screen -- not in effect ATM TODO
             self._pbar.close()
             del self._pbar
+
+
+def keyring_lookup(service_name, username):
+    """
+    Determine a keyring backend to use for storing & retrieving credentials as
+    follows:
+
+    - If the user has specified a backend explicitly via the
+      ``PYTHON_KEYRING_BACKEND`` environment variable or a ``keyringrc.cfg``
+      file, use that backend without checking whether it's usable (If it's not,
+      the user messed up).
+
+    - Otherwise, query the default backend (which is guaranteed to already have
+      the requisite dependencies installed) for the credentials for the given
+      service name and username.  If this completes without error (regardless
+      of whether the backend contains any such credentials), use that backend.
+
+    - If the query fails (e.g., because a GUI is required but the session is in
+      a plain terminal), try using the ``EncryptedKeyring`` backend.
+
+      - If the default backend *was* the ``EncryptedKeyring`` backend, error.
+
+      - If the ``EncryptedKeyring`` backend is not in the list of available
+        backends (likely because its dependencies are not installed, though
+        that shouldn't happen if dandi was installed properly), error.
+
+      - If ``EncryptedKeyring``'s data file already exists, use it as the
+        backend.
+
+      - If ``EncryptedKeyring``'s data file does not already exist, ask the
+        user whether they want to start using ``EncryptedKeyring``.  If yes,
+        then set ``keyringrc.cfg`` (if it does not already exist) to specify it
+        as the default backend, and return the backend.  If no, error.
+
+    Returns a keyring backend and the password it holds (if any) for the given
+    service and username.
+    """
+
+    kb = load_env() or load_config()
+    if kb:
+        return (kb, kb.get_password(service_name, username))
+    kb = get_keyring()
+    try:
+        password = kb.get_password(service_name, username)
+    except KeyringError as e:
+        lgr.info("Default keyring errors on query: %s", e)
+        if isinstance(kb, EncryptedKeyring):
+            lgr.info(
+                "Default keyring is EncryptedKeyring; abandoning keyring procedure"
+            )
+            raise
+        # Use `type(..) is` instead of `isinstance()` to weed out subclasses
+        kbs = [k for k in get_all_keyring() if type(k) is EncryptedKeyring]
+        assert (
+            len(kbs) == 1
+        ), "EncryptedKeyring not available; is pycryptodomex installed?"
+        kb = kbs[0]
+        if op.exists(kb.file_path):
+            lgr.info("EncryptedKeyring file exists; using as keyring backend")
+            return (kb, kb.get_password(service_name, username))
+        lgr.info("EncryptedKeyring file does not exist")
+        if askyesno("Would you like to establish an encrypted keyring?", default=True):
+            keyring_cfg = Path(keyringrc_file())
+            if keyring_cfg.exists():
+                lgr.info("%s exists; refusing to overwrite", keyring_cfg)
+            else:
+                lgr.info(
+                    "Configuring %s to use EncryptedKeyring as default backend",
+                    keyring_cfg,
+                )
+                keyring_cfg.parent.mkdir(parents=True, exist_ok=True)
+                keyring_cfg.write_text(
+                    "[backend]\n"
+                    "default-keyring = keyrings.alt.file.EncryptedKeyring\n"
+                )
+            return (kb, None)
+        raise
+    else:
+        return (kb, password)
+
+
+def keyringrc_file():
+    return op.join(config_root(), "keyringrc.cfg")
